@@ -15,7 +15,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#define LOCAL_DEBUG
+#undef LOCAL_DEBUG
 #undef DO_CLOCKING
 
 #ifdef _WIN32
@@ -1838,7 +1838,7 @@ LOCAL_DEBUG_CALLER_OUT( "text = \"%s\", font = %p, compression = %d", text, font
 	if( !get_text_glyph_map( text, font, &map, attr, length) )
 		return NULL;
 	
-	if( map.width == 0 ) 
+	if( map.width <= 0 ) 
 		return NULL;
 
 	apply_text_3D_type( attr->type, &(offset_3d_x), &(offset_3d_y) );
