@@ -30,9 +30,6 @@
 #include "shape.h"
 #include "canvas.h"
 
-/* we do it here so that we can make it dependant on fewest possible number 
- * of header, so we can reuse it in other apps */
-extern ScreenInfo Scr;
 
 inline Bool
 get_current_canvas_geometry( ASCanvas * pc, int *px, int *py, unsigned int *pwidth, unsigned int *pheight, unsigned int *pbw )
@@ -173,8 +170,8 @@ create_ascanvas (Window w)
 	{
 		pc = safecalloc (1, sizeof (ASCanvas));
 		pc->w = w;
-		LOCAL_DEBUG_CALLER_OUT( "<<#########>>canvas %p created for window %lX", pc, pc->w );
 		refresh_canvas_config (pc);
+		LOCAL_DEBUG_CALLER_OUT( "<<#########>>canvas %p created for w: %lX; geom = %dx%d%+d%+d, bw = %d", pc, pc->w, pc->width, pc->height, pc->root_x, pc->root_y, pc->bw );
 	}
 	return pc;
 }
@@ -188,9 +185,9 @@ create_ascanvas_container (Window w)
 	{
 		pc = safecalloc (1, sizeof (ASCanvas));
 		pc->w = w;
-		LOCAL_DEBUG_CALLER_OUT( "<<#########>>container canvas %p created for window %lX", pc, pc->w );
         pc->state = CANVAS_CONTAINER ;
 		refresh_canvas_config (pc);
+		LOCAL_DEBUG_CALLER_OUT( "<<#########>>container canvas %p created for w: %lX; geom = %dx%d%+d%+d, bw = %d", pc, pc->w, pc->width, pc->height, pc->root_x, pc->root_y , pc->bw);
 	}
 	return pc;
 }
@@ -207,7 +204,8 @@ destroy_ascanvas (ASCanvas ** pcanvas)
 		{
 			destroy_visual_pixmap(ASDefaultVisual, &(pc->saved_canvas));
 			destroy_visual_pixmap(ASDefaultVisual, &(pc->canvas));
-            if (pc->saved_shape)
+			LOCAL_DEBUG_OUT( "saved_shape = %p, shape = %p", pc->saved_shape, pc->shape ); 
+            if (pc->saved_shape && pc->saved_shape != pc->shape)
 				destroy_shape( &(pc->saved_shape));
             if (pc->shape)
 				destroy_shape( &(pc->shape));
@@ -281,6 +279,8 @@ get_canvas_position( ASCanvas *pc, Window *pparent, int *px, int *py, unsigned i
 Bool
 make_canvas_rectangle (ASCanvas * pc, ASImage * im, int x, int y, int *cx, int *cy, int *cwidth, int *cheight)
 {
+	if( pc == NULL || im == NULL || cx == NULL || cy == NULL ||cwidth == NULL || cheight == NULL ) 
+		return False;
 	*cwidth = im->width;
 	*cheight = im->height;
 	*cx = x;
@@ -314,6 +314,7 @@ draw_canvas_image (ASCanvas * pc, ASImage * im, int x, int y)
 	Pixmap        p;
 	int           real_x, real_y;
 	int           width, height;
+	Bool 		  done = False;
 
 	LOCAL_DEBUG_CALLER_OUT ("pc(%p)->im(%p)->x(%d)->y(%d)", pc, im, x, y);
 	if (im == NULL || pc == NULL)
@@ -325,14 +326,22 @@ draw_canvas_image (ASCanvas * pc, ASImage * im, int x, int y)
         return False;
 
     LOCAL_DEBUG_OUT ("drawing image %dx%d at %dx%d%+d%+d", im->width, im->height, width, height, real_x, real_y);
-	if (asimage2drawable (ASDefaultVisual, p, im, ASDefaultDrawGC, real_x - x, real_y - y, real_x, real_y, width, height, True))
+	if( get_flags( ASDefaultVisual->glx_support, ASGLX_UseForImageTx ) )
+		done = asimage2drawable_gl( ASDefaultVisual, p, im, 
+									real_x - x, real_y - y, real_x, real_y,
+        	   						width, height, 
+			   						pc->width, pc->height, 
+									False );
+	if( !done ) 		
+		done = asimage2drawable (ASDefaultVisual, p, im, ASDefaultDrawGC, real_x - x, real_y - y, real_x, real_y, width, height, True);
+
+	if ( done )
 	{
 		set_flags (pc->state, CANVAS_OUT_OF_SYNC);
         XClearArea( dpy, pc->w, real_x, real_y, width, height, True );
         XSync (dpy, False);
-        return True;
 	}
-    return False;
+    return done;
 }
 
 void
@@ -341,6 +350,9 @@ fill_canvas_mask (ASCanvas * pc, int win_x, int win_y, int width, int height)
 	int           real_x, real_y;
 	int           real_width, real_height;
 
+	if( pc == NULL ) 
+		return;
+
     if (pc->shape != None && !get_flags( pc->state, CANVAS_CONTAINER ) )
 	{
 		real_x = MAX (win_x, 0);
@@ -415,8 +427,11 @@ void
 update_canvas_display_mask (ASCanvas * pc, Bool force)
 {
 #ifdef SHAPE
-LOCAL_DEBUG_CALLER_OUT( "canvas(%p)->window(%lx)->canvas_pixmap(%lx)->size(%dx%d)", pc, pc->w, pc->canvas, pc->width, pc->height );
-    if (pc && pc->w != None )
+LOCAL_DEBUG_CALLER_OUT( "canvas(%p)", pc );
+    if( pc == NULL )
+        return ;
+	LOCAL_DEBUG_OUT( "window(%lx)->canvas_pixmap(%lx)->size(%dx%d)", pc->w, pc->canvas, pc->width, pc->height );
+    if ( pc->w != None )
 	{
         if( force || !get_flags( pc->state, CANVAS_CONTAINER ))
         {
@@ -479,7 +494,11 @@ void  trace_update_canvas_display (ASCanvas * pc, const char *file, int line)
 void
 update_canvas_display (ASCanvas * pc)
 {
-LOCAL_DEBUG_CALLER_OUT( "canvas(%p)->window(%lx)->canvas_pixmap(%lx)->size(%dx%d)", pc, pc->w, pc->canvas, pc->width, pc->height );
+	LOCAL_DEBUG_CALLER_OUT( "canvas(%p)", pc );
+    if( pc == NULL )
+        return ;
+
+	LOCAL_DEBUG_OUT( "window(%lx)->canvas_pixmap(%lx)->size(%dx%d)", pc->w, pc->canvas, pc->width, pc->height );
     if (pc && pc->w != None )
 	{
         if( !get_flags( pc->state, CANVAS_CONTAINER ) )
@@ -579,8 +598,12 @@ swap_save_canvas( ASCanvas *pc )
 void
 clear_canvas_shape (ASCanvas * pc, Bool force_for_container)
 {
-LOCAL_DEBUG_CALLER_OUT( "canvas(%p)->window(%lx)->canvas_pixmap(%lx)->size(%dx%d)", pc, pc->w, pc->canvas, pc->width, pc->height );
-    if (pc && pc->w != None )
+	LOCAL_DEBUG_CALLER_OUT( "canvas(%p)", pc );
+	if( pc == NULL )
+        return ;
+
+	LOCAL_DEBUG_OUT( "window(%lx)->canvas_pixmap(%lx)->size(%dx%d)", pc->w, pc->canvas, pc->width, pc->height );
+    if ( pc->w != None )
 	{
         if( !get_flags( pc->state, CANVAS_CONTAINER ) || force_for_container )
         {
@@ -601,10 +624,13 @@ check_canvas_shaped( ASCanvas *pc)
 {
     Bool           boundingShaped= False;
 #ifdef SHAPE
-    int           dumm;
-    unsigned      udumm;
-    XShapeQueryExtents (dpy, pc->w,
-                        &boundingShaped, &dumm, &dumm, &udumm, &udumm, &dumm, &dumm, &dumm, &udumm, &udumm);
+	if( pc ) 
+	{	
+    	int           dumm;
+    	unsigned      udumm;
+    	XShapeQueryExtents (dpy, pc->w,
+                        	&boundingShaped, &dumm, &dumm, &udumm, &udumm, &dumm, &dumm, &dumm, &udumm, &udumm);
+	}
 #endif
     return boundingShaped ;
 }
@@ -802,6 +828,9 @@ resize_canvas (ASCanvas * pc, unsigned int width, unsigned int height)
 {
     ASFlagType changes = 0 ;        
 
+	if( pc == NULL ) 
+		return 0;
+
     LOCAL_DEBUG_CALLER_OUT( "canvas(%p)->window(%lx)->geom(%ux%u)", pc, pc->w, width, height );
     /* Setting background to None to avoid background pixmap tiling
 	 * while resizing */
@@ -836,7 +865,11 @@ ASFlagType
 moveresize_canvas (ASCanvas * pc, int x, int y, unsigned int width, unsigned int height)
 {
     ASFlagType changes = 0 ;
-    LOCAL_DEBUG_CALLER_OUT( "canvas(%p)->window(%lx)->geom(%ux%u%+d%+d)", pc, pc->w, width, height, x, y );
+	
+	if( pc == NULL ) 
+		return 0;
+    
+	LOCAL_DEBUG_CALLER_OUT( "canvas(%p)->window(%lx)->geom(%ux%u%+d%+d)", pc, pc->w, width, height, x, y );
     /* Setting background to None to avoid background pixmap tiling
 	 * while resizing */
 
@@ -873,6 +906,9 @@ moveresize_canvas (ASCanvas * pc, int x, int y, unsigned int width, unsigned int
 void
 move_canvas (ASCanvas * pc, int x, int y)
 {
+	if( pc == NULL ) 
+		return;
+
 LOCAL_DEBUG_CALLER_OUT( "canvas(%p)->window(%lx)->geom(%+d%+d)", pc, pc->w, x, y );
     XMoveWindow (dpy, pc->w, x, y);
 }
@@ -891,8 +927,12 @@ unmap_canvas_window( ASCanvas *pc )
 void
 map_canvas_window( ASCanvas *pc, Bool raised )
 {
-	LOCAL_DEBUG_CALLER_OUT( "pc=%p, raised = %s", pc , raised ? "True":"False" );
-    if( pc && pc->w != None )
+	LOCAL_DEBUG_CALLER_OUT( "pc=%p", pc );
+	if( pc == NULL ) 
+		return;
+
+	LOCAL_DEBUG_OUT( "raised = %s", raised ? "True":"False" );
+    if( pc->w != None )
     {
         if( raised )
             XMapRaised( dpy, pc->w );
@@ -957,7 +997,7 @@ reparent_canvas_window( ASCanvas *pc, Window dst, int x, int y )
 void
 add_canvas_grid( ASGrid *grid, ASCanvas *canvas, int outer_gravity, int inner_gravity, int vx, int vy )
 {
-    if( canvas )
+    if( canvas && grid )
     {
         int x = canvas->root_x + vx ;
         int y = canvas->root_y + vy ;
@@ -972,15 +1012,18 @@ LOCAL_DEBUG_CALLER_OUT( "(%p,%ux%u%+d%+d)", canvas, canvas->width, canvas->heigh
 void
 set_root_clip_area( ASCanvas *canvas )
 {
-    ASDefaultScr->RootClipArea.x = canvas->root_x+(int)canvas->bw;
-    ASDefaultScr->RootClipArea.y = canvas->root_y+(int)canvas->bw;
-    ASDefaultScr->RootClipArea.width  = canvas->width;
-    ASDefaultScr->RootClipArea.height = canvas->height;
-    if( ASDefaultScr->RootImage )
-    {
-        safe_asimage_destroy( ASDefaultScr->RootImage );
-        ASDefaultScr->RootImage = NULL ;
-    }
+	if( canvas )
+	{	
+    	ASDefaultScr->RootClipArea.x = canvas->root_x+(int)canvas->bw;
+    	ASDefaultScr->RootClipArea.y = canvas->root_y+(int)canvas->bw;
+    	ASDefaultScr->RootClipArea.width  = canvas->width;
+    	ASDefaultScr->RootClipArea.height = canvas->height;
+    	if( ASDefaultScr->RootImage )
+    	{
+        	safe_asimage_destroy( ASDefaultScr->RootImage );
+        	ASDefaultScr->RootImage = NULL ;
+    	}
+	}
 }
 
 /*************************************************************************/
