@@ -99,8 +99,18 @@ Atom          _XA_NET_WM_STATE_ABOVE;
 Atom          _XA_NET_WM_STATE_BELOW;
 
 Atom          _XA_NET_WM_PID;
+Atom          _XA_NET_WM_ICON;
 Atom          _XA_NET_WM_PING;
 
+/* Implements KDE System tray specs:
+ * http://developer.kde.org/documentation/library/kdeqt/kde3arch/protocols-docking.html
+ * https://listman.redhat.com/archives/xdg-list/2002-March/msg00014.html
+ * http://standards.freedesktop.org/systemtray-spec/systemtray-spec-0.2.html#ftn.id2494129
+ * 
+ */
+Atom          _XA_KDE_DESKTOP_WINDOW = None ;
+Atom 		  _XA_KDE_NET_SYSTEM_TRAY_WINDOW_FOR = None;
+
 
 /* Crossreferences of atoms into flag value for
    different atom list type of properties :*/
@@ -122,6 +132,9 @@ AtomXref      MainHints[] = {
 	{"_NET_WM_WINDOW_TYPE", &_XA_NET_WM_WINDOW_TYPE},
 	{"_NET_WM_STATE", &_XA_NET_WM_STATE},
 	{"_NET_WM_PID", &_XA_NET_WM_PID},
+	{"_NET_WM_ICON", &_XA_NET_WM_ICON},
+	{"_KDE_DESKTOP_WINDOW", &_XA_KDE_DESKTOP_WINDOW},
+	{"_KDE_NET_SYSTEM_TRAY_WINDOW_FOR", &_XA_KDE_NET_SYSTEM_TRAY_WINDOW_FOR},
 	{NULL, NULL, 0, None}
 };
 
@@ -631,6 +644,37 @@ read_extwm_pid (ASRawHints * hints, Window w)
 	}
 }
 
+void
+read_extwm_icon (ASRawHints * hints, Window w)
+{
+	if (hints && w != None)
+	{
+		if (read_32bit_proplist (w, _XA_NET_WM_ICON, 2+48*48, &(hints->extwm_hints.icon), &(hints->extwm_hints.icon_length)) )
+			set_flags (hints->extwm_hints.flags, EXTWM_ICON);
+	}
+}
+
+void
+read_kde_desktop_window (ASRawHints * hints, Window w)
+{
+	if (hints && w != None)
+	{
+		CARD32 dummy ;
+		if (read_32bit_property (w, _XA_KDE_DESKTOP_WINDOW, &dummy))
+			set_flags( hints->kde_hints.flags, KDE_DesktopWindow);
+	}
+}
+	
+void
+read_kde_systray_window_for (ASRawHints * hints, Window w)
+{
+	if (hints && w != None)
+	{
+		if (read_32bit_property (w, _XA_KDE_NET_SYSTEM_TRAY_WINDOW_FOR, &hints->kde_hints.systray_window_for))
+			set_flags( hints->kde_hints.flags, KDE_SysTrayWindowFor);
+	}
+}
+
 Bool
 default_parent_hints_func (Window parent, ASParentHints * dst)
 {
@@ -717,7 +761,11 @@ HintsDescriptions[] =
 	{&_XA_NET_WM_DESKTOP, read_extwm_desktop, HINT_STARTUP | HINT_PROTOCOL, HINTS_ExtendedWM},
 	{&_XA_NET_WM_WINDOW_TYPE, read_extwm_window_type, HINT_STARTUP | HINT_GENERAL, HINTS_ExtendedWM},
 	{&_XA_NET_WM_STATE, read_extwm_state, HINT_STARTUP | HINT_GENERAL, HINTS_ExtendedWM},
-	{&_XA_NET_WM_PID, read_extwm_pid, HINT_STARTUP, HINTS_ExtendedWM},
+	{&_XA_NET_WM_PID, read_extwm_pid, HINT_GENERAL, HINTS_ExtendedWM},
+	{&_XA_NET_WM_ICON, read_extwm_icon, HINT_GENERAL, HINTS_ExtendedWM},
+	{&_XA_KDE_DESKTOP_WINDOW, read_kde_desktop_window, HINT_GENERAL, HINTS_KDE},
+	{&_XA_KDE_NET_SYSTEM_TRAY_WINDOW_FOR, read_kde_systray_window_for, HINT_GENERAL, HINTS_KDE},
+
 	{
 	NULL, NULL, 0, HINTS_Supported}
 };
@@ -1000,6 +1048,8 @@ print_extwm_hints (stream_func func, void *stream, ExtendedWMHints * hints)
 		func (stream, "EXTWM._NET_WM_DESKTOP = %ld;\n", hints->desktop);
 	if (get_flags (hints->flags, EXTWM_PID))
 		func (stream, "EXTWM._NET_WM_PID = %ld;\n", hints->pid);
+	if (get_flags (hints->flags, EXTWM_ICON))
+		func (stream, "EXTWM._NET_WM_ICON.length = %ld;\n", hints->icon_length);
 
 	print_list_hints (func, stream, hints->flags, EXTWM_WindowType, "EXTWM._NET_WM_WINDOW_TYPE");
 	print_list_hints (func, stream, hints->flags, EXTWM_State, "EXTWM._NET_WM_STATE");
@@ -1109,9 +1159,7 @@ handle_manager_property_update (Window w, Atom property, ASRawHints * raw)
 	if (w && property && raw)
 	{
 		/* Here we are only interested in properties updtaed by the Window Manager : */
-		if (property == _XA_WM_NAME || property == _XA_WM_ICON_NAME ||
-			property == _XA_NET_WM_NAME || property == _XA_NET_WM_ICON_NAME ||
-			property == _XA_NET_WM_VISIBLE_NAME || property == _XA_NET_WM_VISIBLE_ICON_NAME)
+		if (IsNameProp(property))
 		{
 			read_wm_state (raw, w);
 			read_wm_icon_name (raw, w);
@@ -1148,6 +1196,24 @@ handle_manager_property_update (Window w, Atom property, ASRawHints * raw)
 	return False;
 }
 
+Bool 
+get_extwm_state_flags (Window w, ASFlagType *flags)
+{
+	if (flags && w != None)
+	{
+		CARD32         *protocols;
+		long          nprotos = 0;
+
+        if (read_32bit_proplist (w, _XA_NET_WM_STATE, 6, &protocols, &nprotos))
+		{
+			translate_atom_list (flags, EXTWM_State, protocols, nprotos);
+			free (protocols);
+			return True;
+		}
+	}
+	return False;
+}
+
 
 
 /**********************************************************************************/
@@ -1184,6 +1250,11 @@ set_client_state (Window w, struct ASStatusHints *status)
 				extwm_states[used++] = _XA_NET_WM_STATE_SHADED;
 				gnome_state |= WIN_STATE_SHADED;
 			}
+			if (get_flags (status->flags, AS_Hidden))
+			{
+				extwm_states[used++] = _XA_NET_WM_STATE_HIDDEN;
+				gnome_state |= WIN_STATE_HIDDEN;
+			}
 			if (get_flags (status->flags, AS_MaximizedX))
 			{
 				extwm_states[used++] = _XA_NET_WM_STATE_MAXIMIZED_HORZ;
@@ -1194,6 +1265,7 @@ set_client_state (Window w, struct ASStatusHints *status)
 				extwm_states[used++] = _XA_NET_WM_STATE_MAXIMIZED_VERT;
 				gnome_state |= WIN_STATE_MAXIMIZED_VERT;
 			}
+			
 			if (read_32bit_proplist (w, _XA_NET_WM_WINDOW_TYPE, MAX_NET_WM_STATES, &old_extwm_state, &old_extwm_used))
 			{
 				translate_atom_list (&old_state, EXTWM_State, old_extwm_state, old_extwm_used);
@@ -1266,7 +1338,7 @@ set_client_protocols (Window w, ASFlagType protocols, ASFlagType extwm_protocols
 LOCAL_DEBUG_OUT( "protocols=0x%lX", protocols );
     if (w && protocols)
 	{
-		CARD32         *list, *extwm_list;
+		CARD32         *list = NULL, *extwm_list = NULL;
 		long          nitems, extwm_nitems;
 
 		encode_atom_list (&(WM_Protocols[0]), &list, &nitems, protocols);
@@ -1276,18 +1348,17 @@ LOCAL_DEBUG_OUT( "protocols=0x%lX", protocols );
 		if( extwm_nitems > 0 )
 		{
 			int i ;
-			list = realloc( list, sizeof(CARD32)*extwm_nitems );
+			list = realloc( list, sizeof(CARD32)*(nitems+extwm_nitems) );
 			for( i = 0  ; i < extwm_nitems ; ++i )
 				list[nitems+i] = extwm_list[i] ;
 			free( extwm_list );
 			nitems += extwm_nitems ;
 		}	 
 
-        if (nitems > 0)
-		{
+        if (nitems > 0 && list )
 			set_32bit_proplist (w, _XA_WM_PROTOCOLS, XA_ATOM, list, nitems);
+		if( list )
 			free (list);
-		}
 	}
 }
 
@@ -1315,7 +1386,9 @@ set_extwm_hints (Window w, ExtendedWMHints * extwm_hints)
 		if (get_flags (extwm_hints->flags, EXTWM_DESKTOP))
 			set_32bit_property (w, _XA_NET_WM_DESKTOP, XA_CARDINAL, extwm_hints->desktop);
 		if (get_flags (extwm_hints->flags, EXTWM_PID))
-			set_32bit_property (w, _XA_NET_WM_PID, XA_CARDINAL, extwm_hints->pid);
+			set_32bit_property (w, _XA_NET_WM_PID, XA_CARDINAL, extwm_hints->pid); 
+		if (get_flags (extwm_hints->flags, EXTWM_ICON))
+			set_32bit_proplist (w, _XA_NET_WM_ICON, XA_CARDINAL, extwm_hints->icon, extwm_hints->icon_length);
 	}
 }
 
@@ -1343,6 +1416,15 @@ set_client_hints (Window w, XWMHints * hints, XSizeHints * size_hints, ASFlagTyp
 	}
 }
 
+void
+set_client_cmd (Window w)
+{
+	if ( w != None && MyArgsPtr->saved_argv && MyArgsPtr->saved_argc > 0 )
+	{
+		XSetCommand(dpy, w, MyArgsPtr->saved_argv, MyArgsPtr->saved_argc);	
+	}
+}
+
 /***************************************************************************
  * ICCCM Client Messages - Section 4.2.8 of the ICCCM dictates that all
  * client messages will have the following form:
