@@ -75,6 +75,7 @@ struct SaveWindowAuxData
     char  this_host[MAXHOSTNAME];
     FILE *f;
 	ASHashTable *res_name_counts;
+	Bool only_modules ;
 };
 
 int
@@ -130,11 +131,52 @@ char *pid2cmd ( int pid )
 			buf[MAX_CMDLINE_SIZE-1] = '\0' ;
 			cmd = mystrdup( &buf[0] );
 		}		
+	    fclose (f);
 	}
-    fclose (f);
 	return cmd;
 }
 
+static 
+char *filter_out_geometry( char *cmd_args, char **geom_keyword )
+{
+	char *clean = mystrdup(cmd_args); 
+	char *token_start ; 
+	char *token_end = clean;
+	
+	do
+	{
+		token_start = token_end ;
+  
+		if( token_start[0] == '-' ) 
+		{	
+			if( strncmp( token_start, "-g ", 3 ) == 0 ||
+				strncmp( token_start, "-geometry ", 10 ) == 0 ||
+				strncmp( token_start, "--geometry ", 11 ) == 0 )
+			{
+				char *geom_start = token_start ; 
+				if( geom_keyword ) 
+					*geom_keyword = tokencpy (geom_start);
+		 		token_end = tokenskip( token_start, 1 );
+				if( token_end > token_start ) 
+				{	
+					token_start = token_end ;
+					token_end = parse_geometry(token_start, NULL, NULL, NULL, NULL, NULL ); 
+					if( token_end > token_start ) 
+					{
+						int i = 0; 
+						for( i = 0 ; token_end[i] != '\0' ; ++i)
+							geom_start[i] = token_end[i] ;
+						geom_start[i]	= '\0' ;
+					}
+				}
+				break;
+			}
+		}
+		token_end = tokenskip( token_start, 1 );
+	}while( token_end > token_start );
+	return clean;
+}	 
+
 Bool
 make_aswindow_cmd_iter_func(void *data, void *aux_data)
 {
@@ -142,9 +184,18 @@ make_aswindow_cmd_iter_func(void *data, void *aux_data)
     ASWindow *asw = (ASWindow*)data ;
 	LOCAL_DEBUG_OUT( "window \"%s\", is a %smodule", ASWIN_NAME(asw), ASWIN_HFLAGS(asw,AS_Module)?
 	" ":"non ");
-    if( asw && swad && !ASWIN_HFLAGS(asw,AS_Module) )
+    if( asw && swad )
     {
 		Bool same_host = (asw->hints->client_host == NULL || mystrcasecmp( asw->hints->client_host, swad->this_host )== 0);
+
+		/* don't want to save modules - those are started from autoexec anyways */
+		if( ASWIN_HFLAGS(asw,AS_Module) ) 
+		{	
+			if( !swad->only_modules	)
+				return True;
+		}else if( swad->only_modules	)
+			return True;
+
 		if( asw->hints->client_cmd == NULL && same_host )
 		{
 		 	if( ASWIN_HFLAGS(asw, AS_PID) && asw->hints->pid > 0 )
@@ -162,10 +213,11 @@ make_aswindow_cmd_iter_func(void *data, void *aux_data)
 			int app_no = get_res_name_count( swad->res_name_counts, asw->hints->res_name );
 			char *rname = asw->hints->res_name?asw->hints->res_name:"*" ;
 			char *rclass = asw->hints->res_class?asw->hints->res_class:"*" ;
-			char *name = get_flags( asw->internal_flags, ASWF_NameChanged )?NULL:ASWIN_NAME(asw);
+			char *name = ASWIN_NAME(asw);
 			int i = 0;
 			char *app_name = "*" ;
 			char *cmd_app = NULL, *cmd_args ;
+			Bool supports_geometry = False ;
 			/* need to check if we can use window name in the pattern. It has to be :
 				* 1) Not changed since window initial mapping
 				* 2) all ASCII
@@ -175,6 +227,14 @@ make_aswindow_cmd_iter_func(void *data, void *aux_data)
 				*/
 			if( name == rname || name == rclass ) 
 				name = NULL ;
+			else if( name != NULL && get_flags( asw->internal_flags, ASWF_NameChanged ) ) 
+			{/* allow changed names only for terms, as those could launch sub app inside */
+				int rclass_len = rclass?strlen(rclass):0;
+				if( rclass_len != 5 )
+					name = NULL ;
+				else if( mystrcasecmp( rclass+1, "term" ) != 0 ) 	  
+					name = NULL ;
+			}
 			if(	name )
 			{
 				while( name[i] != '\0' )
@@ -209,25 +269,76 @@ make_aswindow_cmd_iter_func(void *data, void *aux_data)
 			cmd_args = parse_token(asw->hints->client_cmd, &cmd_app );
 			if( cmd_app ) 
 			{   /* we want -geometry to be the first arg, so that terms could correctly launch app with -e arg */
-				fprintf( swad->f, 	"\tExec \"I:%s\" %s -geometry %s %s &\n", app_name, cmd_app, geom, cmd_args );
+				char *geometry_keyword = NULL ; 
+				char *clean_cmd_args = filter_out_geometry( cmd_args, &geometry_keyword ) ;
+				if( geometry_keyword == NULL ) 
+					geometry_keyword = mystrdup(ASWIN_HFLAGS(asw,AS_Module)?"--geometry":"-geometry"); 
+				else
+					supports_geometry = True ;
+
+				fprintf( swad->f, 	"\tExec \"I:%s\" %s %s %s %s &\n", app_name, cmd_app, geometry_keyword, geom, clean_cmd_args );
+				free( clean_cmd_args );
+				free( geometry_keyword );
 				free( cmd_app );	
 			}else
-	            fprintf( swad->f, 	"\tExec \"I:%s\" %s -geometry %s &\n", app_name, asw->hints->client_cmd, geom );
-			fprintf( swad->f, 	"\tWait \"I:%s\" DefaultGeometry %s"
-						        ", Layer %d"
-								", %s"
-								", StartsOnDesk %d"
-								", ViewportX %d"
-								", ViewportY %d"
-								", %s"
-								"\n",
-						     	app_name, pure_geometry,
-							 	ASWIN_LAYER(asw),
-							 	ASWIN_GET_FLAGS(asw,AS_Sticky)?"Sticky":"Slippery",
-								ASWIN_DESK(asw),
-								asw->status->viewport_x,
-								asw->status->viewport_y,
-								ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
+	            fprintf( swad->f, 	"\tExec \"I:%s\" %s %s %s &\n", 
+									app_name, asw->hints->client_cmd, ASWIN_HFLAGS(asw,AS_Module)?"--geometry":"-geometry", geom );
+			if( ASWIN_HFLAGS(asw,AS_Module) ) 
+			{
+				fprintf( swad->f, "\tWait \"I:%s\" Layer %d\n", app_name, ASWIN_LAYER(asw));
+			}else if( ASWIN_GET_FLAGS(asw,AS_Sticky) )
+			{
+				if( supports_geometry ) 	  
+					fprintf( swad->f, 	"\tWait \"I:%s\" Layer %d"
+										", Sticky"
+										", StartsOnDesk %d"
+										", %s"
+										"\n",
+						     			app_name, ASWIN_LAYER(asw),
+										ASWIN_DESK(asw),
+										ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
+				else
+					fprintf( swad->f, 	"\tWait \"I:%s\" DefaultGeometry %s"
+							        	", Layer %d"
+										", Sticky"
+										", StartsOnDesk %d"
+										", %s"
+										"\n",
+						     			app_name, pure_geometry,
+							 			ASWIN_LAYER(asw),
+										ASWIN_DESK(asw),
+										ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
+			}else
+			{
+				if( supports_geometry ) 
+					fprintf( swad->f, 	"\tWait \"I:%s\" Layer %d"
+										", Slippery"
+										", StartsOnDesk %d"
+										", ViewportX %d"
+										", ViewportY %d"
+										", %s"
+										"\n",
+						     			app_name, ASWIN_LAYER(asw),
+										ASWIN_DESK(asw),
+										((asw->status->x + asw->status->viewport_x) / Scr.MyDisplayWidth)*Scr.MyDisplayWidth,
+										((asw->status->y + asw->status->viewport_y) / Scr.MyDisplayHeight)*Scr.MyDisplayHeight,
+										ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
+				else										   
+					fprintf( swad->f, 	"\tWait \"I:%s\" DefaultGeometry %s"
+							        	", Layer %d"
+										", Slippery"
+										", StartsOnDesk %d"
+										", ViewportX %d"
+										", ViewportY %d"
+										", %s"
+										"\n",
+						     			app_name, pure_geometry,
+							 			ASWIN_LAYER(asw),
+										ASWIN_DESK(asw),
+										((asw->status->x + asw->status->viewport_x) / Scr.MyDisplayWidth)*Scr.MyDisplayWidth,
+										((asw->status->y + asw->status->viewport_y) / Scr.MyDisplayHeight)*Scr.MyDisplayHeight,
+										ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
+			}	 
 			if( pure_geometry ) 
 				free( pure_geometry );
 			if( geom )
@@ -273,6 +384,13 @@ save_aswindow_list( ASWindowList *list, char *file )
 	if( swad.f )
     {
 		fprintf( swad.f, "Function \"WorkspaceState\"\n" );
+		swad.only_modules = False ;
+		swad.res_name_counts = create_ashash(0, string_hash_value, string_compare, string_destroy);
+        iterate_asbidirlist( list->clients, make_aswindow_cmd_iter_func, &swad, NULL, False );
+		destroy_ashash( &(swad.res_name_counts) );
+		fprintf( swad.f, "EndFunction\n\n" );
+		fprintf( swad.f, "Function \"WorkspaceModules\"\n" );
+		swad.only_modules = True ;
 		swad.res_name_counts = create_ashash(0, string_hash_value, string_compare, string_destroy);
         iterate_asbidirlist( list->clients, make_aswindow_cmd_iter_func, &swad, NULL, False );
 		destroy_ashash( &(swad.res_name_counts) );
@@ -750,6 +868,7 @@ delist_aswindow( ASWindow *t )
 	publish_aswindow_list( Scr.Windows, False );	   
 }
 
+#if 0
 void
 update_visibility( int desk )
 {
@@ -826,6 +945,7 @@ update_visibility( int desk )
         ASCanvas *frame = asws[i]->frame_canvas ;
 		int client_left, client_right, client_bottom, client_top ; 
         int r ;
+		Bool visible = False;
 
         if( ASWIN_GET_FLAGS( asws[i], AS_Iconic ) )
         {
@@ -835,7 +955,7 @@ update_visibility( int desk )
         }else if( ASWIN_GET_FLAGS( asws[i], AS_Shaded ) )
             client = frame ;
 
-        ASWIN_CLEAR_FLAGS( asws[0], AS_Visible );
+        ASWIN_CLEAR_FLAGS( asws[0], AS_Hidden );
 
         r = VECTOR_USED(*rects);
 		client_left = client->root_x ;
@@ -849,12 +969,15 @@ update_visibility( int desk )
                 client_bottom > vrect[r].y &&
                 client_top    < vrect[r].y+(int)vrect[r].height )
             {
-                ASWIN_SET_FLAGS( asws[0], AS_Visible );
+				visible = True ;
                 break;
             }
         }
+		if( !visible ) 
+			ASWIN_SET_FLAGS( asws[0], AS_Hidden );
     }
 }
+#endif
 
 void
 restack_window_list( int desk, Bool send_msg_only )
@@ -903,7 +1026,7 @@ restack_window_list( int desk, Bool send_msg_only )
         ASWindow **members = PVECTOR_HEAD(ASWindow*,l[i]->members);
         if( end_k > ids->allocated )
             end_k = ids->allocated ;
-        LOCAL_DEBUG_OUT( "layer %ld, end_k = %d", i, end_k );
+        LOCAL_DEBUG_OUT( "layer[%ld] = %d, end_k = %d", i, l[i]->layer, end_k );
         for( k = 0 ; k < end_k ; k++ )
         {
             register ASWindow *asw = members[k] ;
@@ -933,12 +1056,15 @@ restack_window_list( int desk, Bool send_msg_only )
         {
             register int k, end_k = PVECTOR_USED(l[i]->members) ;
             register ASWindow **members = PVECTOR_HEAD(ASWindow*,l[i]->members);
-            if( end_k > ids->allocated )
-                end_k = ids->allocated ;
-            LOCAL_DEBUG_OUT( "layer %ld, end_k = %d", i, end_k );
+            if( windows_num+end_k > ids->allocated )
+                end_k = ids->allocated - windows_num ;
+            LOCAL_DEBUG_OUT( "layer[%ld] = %d, end_k = %d", i, l[i]->layer, end_k );
             for( k = 0 ; k < end_k ; k++ )
                 if( ASWIN_DESK(members[k]) == desk && !ASWIN_GET_FLAGS(members[k], AS_Dead))
+				{
                     windows[windows_num++] = get_window_frame(members[k]);
+					LOCAL_DEBUG_OUT( "\t windows[%d] = %lX", k, windows[windows_num-1] );
+				}
         }
 
         LOCAL_DEBUG_OUT( "Setting stacking order: windows_num = %ld, ", windows_num );
@@ -1214,6 +1340,7 @@ hide_focus()
     Scr.Windows->focused = NULL;
     Scr.Windows->ungrabbed = NULL;
     XRaiseWindow(dpy, Scr.ServiceWin);
+	LOCAL_DEBUG_OUT( "XSetInputFocus(window= %lX (service_win), time = %lu)", Scr.ServiceWin, Scr.last_Timestamp);
     XSetInputFocus (dpy, Scr.ServiceWin, RevertToParent, Scr.last_Timestamp);
     XSync(dpy, False );
 }
@@ -1264,10 +1391,20 @@ focus_window( ASWindow *asw, Window w )
     LOCAL_DEBUG_OUT( "focusing window %lX, client %lX, frame %lX, asw %p", w, asw->w, asw->frame, asw );
 	/* using last_Timestamp here causes problems when moving between screens */
 	/* at the same time using CurrentTime all the time seems to cause some apps to fail,
-	 * most noticeably GTK-perl */
+	 * most noticeably GTK-perl 
+	 * 
+	 * Take 2: disabled CurrentTime altogether as it screwes up focus handling
+	 * Basically if you use CurrentTime when there are still bunch of Events 
+	 * in the queue, those evens will not have any effect if you try setting 
+	 * focus using their time, as X aready used its own friggin current time.
+	 * Don't ask, its a mess.
+	 * */
 	if( w != None )
-	    XSetInputFocus ( dpy, w, RevertToParent, 
-						(Scr.Windows->focused == NULL)?CurrentTime:Scr.last_Timestamp);
+	{
+		Time t = /*(Scr.Windows->focused == NULL)?CurrentTime:*/Scr.last_Timestamp ;
+		LOCAL_DEBUG_OUT( "XSetInputFocus(window= %lX, time = %lu)", w, t);	  
+	    XSetInputFocus ( dpy, w, RevertToParent, t );
+	}
 
     ASSync(False );
     return (w!=None);
@@ -1504,12 +1641,18 @@ LOCAL_DEBUG_OUT("checking i(%d)->end_i(%d)->dir(%d)->AutoReverse(%d)", i, end_i,
 /********************************************************************************/
 /* window list menus regeneration :                                             */
 /********************************************************************************/
-static int
-make_desk_winlist_menu_strcmp(const void *a, const void *b) {
-	FunctionData *fda = *(FunctionData **)a;
-	FunctionData *fdb = *(FunctionData **)b;
-
-	return strcmp(fda->name ? fda->name : "", fdb->name ? fdb->name : "");
+static inline void
+ASWindow2func_data( FunctionCode func, ASWindow *asw, FunctionData *fdata, char *scut, Bool icon_name ) 
+{
+    fdata->func = F_RAISE_IT;
+    fdata->name = mystrdup(icon_name ? ASWIN_ICON_NAME(asw) : ASWIN_NAME(asw));
+	if( !icon_name )
+	 	fdata->name_encoding = ASWIN_NAME_ENCODING(asw) ;
+    fdata->func_val[0] = (long) asw;
+    fdata->func_val[1] = (long) asw->w;
+	if (++(*scut) == ('9' + 1))
+		(*scut) = 'A';		/* Next shortcut key */
+    fdata->hotkey = (*scut);
 }
 
 MenuData *
@@ -1517,79 +1660,67 @@ make_desk_winlist_menu(  ASWindowList *list, int desk, int sort_order, Bool icon
 {
     char menu_name[256];
     MenuData *md ;
+    MenuDataItem *mdi ;
     FunctionData  fdata ;
     char          scut = '0';                  /* Current short cut key */
+    ASWindow **clients;
+    int i, max_i;
 
     if( list == NULL )
         return NULL;;
+    
+	clients = PVECTOR_HEAD(ASWindow*,list->circulate_list);
+    max_i = PVECTOR_USED(list->circulate_list);
 
     if( IsValidDesk( desk ) )
         sprintf( menu_name, "Windows on Desktop #%d", desk );
     else
         sprintf( menu_name, "Windows on All Desktops" );
 
-    if( (md = CreateMenuData (menu_name)) == NULL )
+    if( (md = create_menu_data (menu_name)) == NULL )
         return NULL;
 
-    purge_menu_data_items( md );
-
     memset(&fdata, 0x00, sizeof(FunctionData));
     fdata.func = F_TITLE ;
     fdata.name = mystrdup(menu_name);
     add_menu_fdata_item( md, &fdata, NULL, NULL );
 
-    if( sort_order == ASO_Circulation )
-    {
-        ASWindow **clients = PVECTOR_HEAD(ASWindow*,list->circulate_list);
-        int i = -1, max_i = PVECTOR_USED(list->circulate_list);
-        while( ++i < max_i )
+	if( sort_order == ASO_Alpha ) 
+	{
+		FunctionData **menuitems = safecalloc(sizeof(FunctionData *), max_i);
+		int numitems = 0;
+        
+		for( i = 0 ; i < max_i ; ++i ) 
+		{
+            if ((ASWIN_DESK(clients[i]) == desk || !IsValidDesk(desk)) && !ASWIN_HFLAGS(clients[i], AS_SkipWinList)) 
+			{
+				menuitems[numitems] = safecalloc(1, sizeof(FunctionData));
+				ASWindow2func_data( F_RAISE_IT, clients[i], menuitems[numitems], &scut, icon_name ); 
+				++numitems;
+            }
+        }
+		qsort(menuitems, numitems, sizeof(FunctionData *), compare_func_data_name);
+		for( i = 0 ; i < numitems ; ++i ) 
+		{
+			if( (mdi = add_menu_fdata_item( md, menuitems[i], NULL, 
+								 get_flags( Scr.Feel.flags, WinListHideIcons) ? NULL : 
+								 get_client_icon_image(ASDefaultScr, clients[i]->hints))) != NULL )
+				set_flags( mdi->flags, MD_ScaleMinipixmapDown );
+			safefree(menuitems[i]); /* scrubba-dub-dub */
+		}
+		safefree(menuitems);
+    } else /* if( sort_order == ASO_Circulation || sort_order == ASO_Stacking ) */
+	{
+        for( i = 0 ; i < max_i ; ++i ) 
         {
             if ((ASWIN_DESK(clients[i]) == desk || !IsValidDesk(desk)) && !ASWIN_HFLAGS(clients[i], AS_SkipWinList))
 			{
-                fdata.func = F_RAISE_IT ;
-                fdata.name = mystrdup(icon_name? ASWIN_ICON_NAME(clients[i]) : ASWIN_NAME(clients[i]));
-				if( !icon_name )
-	 				fdata.name_encoding = ASWIN_NAME_ENCODING(clients[i]) ;
-				LOCAL_DEBUG_OUT( "item's encoding = %d, name = \"%s\"", fdata.name_encoding,
-				fdata.name );
-                fdata.func_val[0] = (long) clients[i];
-                fdata.func_val[1] = (long) clients[i]->w;
-		if (++scut == ('9' + 1))
-			scut = 'A';		   /* Next shortcut key */
-                fdata.hotkey = scut;
-                add_menu_fdata_item( md, &fdata, NULL, get_flags( Scr.Feel.flags, WinListHideIcons)? NULL : get_window_icon_image(clients[i]));
+				ASWindow2func_data( F_RAISE_IT, clients[i], &fdata, &scut, icon_name ); 
+                if( (mdi = add_menu_fdata_item( md, &fdata, NULL, get_flags( Scr.Feel.flags, WinListHideIcons)? NULL : 
+									 get_client_icon_image( ASDefaultScr, clients[i]->hints))) != NULL ) 
+					set_flags( mdi->flags, MD_ScaleMinipixmapDown );									 	
             }
         }
-    } else if( sort_order == ASO_Alpha ) {
-        ASWindow **clients = PVECTOR_HEAD(ASWindow*,list->circulate_list);
-        int i = -1, max_i = PVECTOR_USED(list->circulate_list);
-	FunctionData **menuitems = safecalloc(sizeof(FunctionData *), max_i);
-    	FunctionData *sfdata;
-	int numitems = 0;
-        while( ++i < max_i ) {
-            if ((ASWIN_DESK(clients[i]) == desk || !IsValidDesk(desk)) && !ASWIN_HFLAGS(clients[i], AS_SkipWinList)) {
-		sfdata = safecalloc(1, sizeof(FunctionData));
-                sfdata->func = F_RAISE_IT;
-                sfdata->name = mystrdup(icon_name ? ASWIN_ICON_NAME(clients[i]) : ASWIN_NAME(clients[i]));
-				if( !icon_name )
-	 				sfdata->name_encoding = ASWIN_NAME_ENCODING(clients[i]) ;
-                sfdata->func_val[0] = (long) clients[i];
-                sfdata->func_val[1] = (long) clients[i]->w;
-		if (++scut == ('9' + 1))
-			scut = 'A';		/* Next shortcut key */
-                sfdata->hotkey = scut;
-		menuitems[numitems++] = sfdata;
-            }
-        }
-	qsort(menuitems, numitems, sizeof(FunctionData *), make_desk_winlist_menu_strcmp);
-	i = 0;
-	while (i < numitems) {
-		add_menu_fdata_item( md, menuitems[i], NULL, get_flags( Scr.Feel.flags, WinListHideIcons) ? NULL : get_window_icon_image(clients[i]));
-		safefree(menuitems[i++]); /* scrubba-dub-dub */
-	}
-	safefree(menuitems);
-    } else if( sort_order == ASO_Stacking ) {
-    } else {
     }
     return md;
 }
