@@ -365,7 +365,8 @@ LOCAL_DEBUG_OUT( "--DESTR Client(%lx(%s))->CLIENT->canvas(%p)->window(%lx)", asw
          * since that would cause a transition to the Withdrawn state.
 		 */
 #if 1
-        if( !ASWIN_GET_FLAGS(asw, AS_Dead) && get_parent_window( w ) == asw->frame )
+        if( (asw->status == NULL || !get_flags(asw->status->flags, AS_Dead)) && 
+			get_parent_window( w ) == asw->frame )
         {
             if( get_flags( AfterStepState, ASS_Shutdown ) )
                 quietly_reparent_window( w, Scr.Root, xwc.x, xwc.y, AS_CLIENT_EVENT_MASK );
@@ -423,6 +424,8 @@ LOCAL_DEBUG_OUT( "++CREAT Client(%lx(%s))->ICON->canvas(%p)->window(%lx)", asw->
     {                                          /* destroy canvas here */
         w = canvas->w ;
 LOCAL_DEBUG_OUT( "--DESTR Client(%lx(%s))->ICON->canvas(%p)->window(%lx)", asw->w, ASWIN_NAME(asw)?ASWIN_NAME(asw):"noname", canvas, canvas->w );
+		if( asw->icon_title_canvas == canvas ) 
+			asw->icon_title_canvas = NULL ;
         destroy_ascanvas( &canvas );
         if( asw->hints && asw->hints->icon.window == w )
             unregister_aswindow( w );
@@ -896,6 +899,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
     check_icon_canvas( asw, (ASWIN_HFLAGS( asw, AS_Icon) && !get_flags(Scr.Feel.flags, SuppressIcons)) );
 
 	/* 4) we need to prepare icon title window : */
+	LOCAL_DEBUG_CALLER_OUT( "checking icon title canvas(%p)", asw->icon_title_canvas );
 	check_icon_title_canvas( asw, (ASWIN_HFLAGS( asw, AS_IconTitle) &&
                                    get_flags(Scr.Feel.flags, IconTitle) &&
 								  !get_flags(Scr.Feel.flags, SuppressIcons)),
@@ -995,8 +999,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
 	    add_astbar_label( asw->icon_title, 0, 0, 0, frame->title_align,
 							DEFAULT_TBAR_HSPACING, DEFAULT_TBAR_VSPACING,
 						    ASWIN_ICON_NAME(asw),
-							(asw->hints->icon_name_idx < 0)?AS_Text_ASCII :
-							asw->hints->names_encoding[asw->hints->icon_name_idx]);
+							ASWIN_ICON_NAME_ENC(asw));
 	    set_astbar_style( asw->icon_title, BAR_STATE_FOCUSED, AS_ICON_TITLE_MYSTYLE );
   	    set_astbar_style( asw->icon_title, BAR_STATE_UNFOCUSED, AS_ICON_TITLE_UNFOCUS_MYSTYLE );
     }
@@ -1129,7 +1132,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
 			{
 				ASCanvas *canvas = ASWIN_HFLAGS(asw, AS_VerticalTitle)?asw->frame_sides[FR_W]:asw->frame_sides[FR_N];
 	        	/* label ( goes on top of above pixmap ) */
-	        	if( change_astbar_first_label( asw->tbar, asw->hints->names[0], asw->hints->names_encoding[0] ) )
+	        	if( change_astbar_first_label( asw->tbar, ASWIN_NAME(asw), ASWIN_NAME_ENCODING(asw) ) )
   		        	if( canvas )
       		    	{
 	                	render_astbar( asw->tbar, canvas );
@@ -1202,7 +1205,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
                           tbar_layout_row[ASO_TBAR_ELEM_LBL],
                           od->flip,
                           title_align, frame->title_h_spacing, frame->title_v_spacing,
-                          asw->hints->names[0], asw->hints->names_encoding[0]);
+                          ASWIN_NAME(asw), ASWIN_NAME_ENCODING(asw));
 		
 			/* all the buttons go after the label to be rendered over it */
 			/* left buttons : */
